<?php
/**
 * authuserfederated.php
 *
 * PMWiki authuser.php extension for federated authentication
 *
 * Currently supports authentication trough google or any
 * standard OpenID provider (willing to give user firstname and lastname)
 *
 * To add a login with google button to your Site.LoginForm or any other page use
 * <code>
 *  (:google_loginbox:)
 * </code>
 *
 * or
 *
 * <code>
 *  (:openid_loginbox:)
 * </code>
 *
 * Additionally you need to ad the Google/OpenID auth to SiteAdmin.AuthUser
 * <code>
 *  google://module
 * </code>
 *
 * The extension depends on Mewp's openid.php library, and requires Curl and PHP 5.
 *
 * @author Ville Takanen
 * @copyright Copyright (c) 2010, Ville Takanen
 * @license http://www.opensource.org/licenses/mit-license.php MIT
 * @version 20100628
 *
 * Original open-id extension for pmwiki (by Michael Novak) was great help when
 * starting to write this extension.
 */


//Add recipe info to pmwiki
$RecipeInfo['FAuth']['Version'] = '20100628';
//Extension sanity checks:
if (!defined('PmWiki')) exit();
if ($EnableAuthUser == 1){
  echo 'Error in Cookbook fauth.php: Authuser extensions like fauth.php must be included before authuser.php';
  exit;
}
//Require mit-licenced lib for open id
require_once 'openid.php';


//Define markup for federated auth buttons
//Usage of the markup should be pretty straightforward
Markup("google_loginbox", "directives", '/\\(:google_loginbox:\\)/ei', "GoogleLoginBox()");
Markup("openid_loginbox", "directives", '/\\(:openid_loginbox:\\)/ei', "OpenIDLoginBox()");

//Add the function to validate authentication (enabled from SiteAdmin.AuthUser)
$AuthUserFunctions['google'] = 'AuthUserOpenID';
$AuthUserFunctions['openid'] = 'AuthUserOpenID';

//Do we have open-id response as a request?
if(isset($_REQUEST['openid_mode']) || isset($_REQUEST['openid.mode'])){
    //We have signal (and open id)
    //If all is well -> push the id to @_POST for authuser
    //(OpenID is not really reliable when selecting between post/get requests)
    if ($_REQUEST['openid_mode'] == 'id_res' || $_REQUEST['openid.mode'] == 'id_res'){
      $openid = new LightOpenID;
      //(reduntant validation as is done again at login)
      //if($openid->validate()){

        $attr=$openid->getAttributes();

        //@TODO:Should have option in local/nofig.php etc. to set required params
        if(!empty($attr['namePerson/first']) && !empty($attr['namePerson/last'])){
          $_POST['authid'] = $attr['namePerson/first'].$attr['namePerson/last'];
        }
        else{
          //echo "hit4";
          $_POST['authid']=getIdentityAsAuthID();
        }
      //End reduntant validation
      //}
    }
} //No -> maybe its an login relay request then?
else{
  //A login attempt with google login -> try to login trough google
  if(isset($_REQUEST['google_login'])) {
    $openid = new LightOpenID;
    $openid->required = array('namePerson/first','namePerson/last', 'contact/email','contact/country/home');

    //Google auth uses uniform address (and medles with cookies)
    $openid->identity = 'https://www.google.com/accounts/o8/id';
    header('Location: ' . $openid->authUrl());
  }
  //A login attempt with standard OpenID provider -> try login with given id
  else if(isset($_REQUEST['openid_login'])){
    $openid = new LightOpenID;
    $openid->required = array('namePerson/first','namePerson/last', 'contact/email','contact/country/home');

    //Normally we can just use the id user gave us!
    $openid->identity =  $openid->identity = $_REQUEST['openid_identifier'];
    header('Location: ' . $openid->authUrl());
  }
}



/**
 *Loginbox for Google
 */
function GoogleLoginBox(){
  if ($_REQUEST['action']>'') $action='&action='.$_REQUEST['action'];
	$output = '<form action="?google_login'.$action.'" method="post"><button>Login with Google</button></form>';
	return $output;
}

/**
 *Loginbox for standard provider
 */
function OpenIDLoginBox(){
  if ($_REQUEST['action']>'') $action='&action='.$_REQUEST['action'];
	$output = '<form action="?openid_login'.$action.'" method="post"><input name="openid_identifier" type="text" size="30"/><button>Login with OpenID</button></form>';
	return $output;
}

/**
 * This function checks login from request parameters
 */
function AuthUserOpenID($pagename, $id, $pw, $pwlist) {
  try{
    $openid = new LightOpenID;

    //Check if we can validate the session from OpenID provider
    if($openid->validate()){
      return true;
    }

    //nope, return false
    return false;
  } catch(ErrorException $e) {
    return false;
  }
}

/**
 *Convert and identity to an authid
 */
function getIdentityAsAuthID(){

  $original=$_REQUEST['openid.identity'];
  if (empty($original)) $original=$_REQUEST['openid_identity'];

  $fixed=str_replace("~", "-", $original);
  $fixed=str_replace("https://", "",$fixed);
  $fixed=str_replace("http://", "",$fixed);
  $fixed=str_replace("/", "-",$fixed);

  return $fixed;
}


